"""
/*************************************************************************
 *
 *  Project
 *                        __  __ _______ _____  _  __
 *                       |  \/  |__   __|  __ \| |/ /
 *  ___  _ __   ___ _ __ | \  / |  | |  | |__) | ' /
 * / _ \| '_ \ / _ \ '_ \| |\/| |  | |  |  ___/|  <
 *| (_) | |_) |  __/ | | | |  | |  | |  | |    | . \ 
 * \___/| .__/ \___|_| |_|_|  |_|  |_|  |_|    |_|\_\ 
 *      | |
 *      |_|
 *
 *
 * Copyright (C) Akiel Aries, <akiel@akiel.org>, et al.
 *
 * This software is licensed as described in the file LICENSE, which
 * you should have received as part of this distribution. The terms
 * among other details are referenced in the official documentation
 * seen here : https://akielaries.github.io/openMTPK/ along with
 * important files seen in this project.
 *
 * You may opt to use, copy, modify, merge, publish, distribute
 * and/or sell copies of the Software, and permit persons to whom
 * the Software is furnished to do so, under the terms of the
 * LICENSE file. As this is an Open Source effort, all implementations
 * must be of the same methodology.
 *
 *
 *
 * This software is distributed on an AS IS basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied.
 *
 ************************************************************************/

openMTPK Linear Algebra Module
==============================
since the Matrix class is a template class, there are classes created
for both Doubles and Integers

The template Matrix class and mtx struct were converted to type double
methods instead of declaring a method for each type.
    *NOTE* there is a respective 'INTEGER' version of each method that
    defaulted to type double, my recommendation is to use the default
    methods that converted to type double as data will more than likely
    follow the scheme.

    SCHEMES:
        class Matrix (int) -> class MatrixI
        an INTEGER type mtx struct is not possible given floating
        point requirements for certain cases of cassert

FEATURES :
    CLASS : Matrix
        - The default Matrix Class defaulting to floating type
        - MatrixI Class features same methods but Integer types
    METHODS :
        mult
        scalar_mult
        mult_elem
        sqr_err
        add
        scalar_add
        sub
        transpose
        sum
        mean
        concatenate
        diag
        print_shape
        print_mtx
        fill_index

    CLASS : mtx
        - Closely coupled with the Matrix Class
        - mtxI Class features same methods but Integer types
    METHODS :
        zeros
        ones
        randn
        rand

    TODO :
        - Vectors Class? Matrix Class is capable of creating Vectors
        but possible error prone to not fixing to a dimension size.

        - Tensor Class?

    USAGE :
        >> import openmtpk
        >> # create matrix of 3,4 size
        >> mtx = openmtpk.linalg.Matrix(3, 4)
        OR
        >> from openmtpk import linalg
"""

from openmtpk.linalg.linalg import (
    Matrix,
    mtx,
    Vectors,
)

__all__: list[str]
__path__: list[str]



"""
Main entrypoint for building the openMTPK Python API.
Leverages SWIG, setuptools, and a few other dependencies.
"""

# from setuptools import setup, find_packages
import setuptools
from setuptools import setup, dist
from setuptools.extension import Extension
from setuptools.command.install import install
from setuptools.command.build_ext import build_ext
import os
import subprocess
import platform
import urllib.request
import json
import versioneer

this_dir = os.path.dirname(os.path.abspath(__file__))

# arithmetic module related information
arithmetic = Extension(
    'openmtpk.arithmetic._arithmetic',
    include_dirs=[os.path.join(this_dir, 'include/arithmetic')],
    sources=[
        'openmtpk/arithmetic/arithmetic.i',
        'modules/arithmetic/arith.cpp'],
    extra_compile_args=['-std=c++2a'],
    swig_opts=['-Wall', '-c++'],
)

# calculus module related information
calculus = Extension(
    'openmtpk.calculus._calculus',
    include_dirs=[os.path.join(this_dir, 'include/calculus')],
    sources=[
        'openmtpk/calculus/calculus.i',
        'modules/calculus/deriv.cpp'],
    extra_compile_args=['-std=c++2a'],
    swig_opts=['-c++'],
)

# machine learning module related information
ml = Extension(
    'openmtpk.ml._ml',
    include_dirs=[os.path.join(this_dir, 'include/ml')],
    sources=[
        'openmtpk/ml/ml.i',
        'modules/ml/linreg.cpp'],
    extra_compile_args=['-std=c++2a'],
    swig_opts=['-c++'],
)

# linear algebra module related information
linalg = Extension(
    'openmtpk.linalg._linalg',
    include_dirs=[os.path.join(this_dir, 'include/linalg')],
    sources=[
        'openmtpk/linalg/linalg.i',
        'modules/linalg/lao.cpp'],
    extra_compile_args=['-std=c++2a'],
    swig_opts=['-c++'],
)

# number_theory module related information
nt = Extension(
    'openmtpk.nt._nt',
    include_dirs=[os.path.join(this_dir, 'include/nt')],
    sources=[
        'openmtpk/nt/nt.i',
        'modules/arithmetic/arith.cpp',
        'modules/nt/primes.cpp',
        'modules/nt/rc4.cpp',
        'modules/nt/cipher.cpp'],
    extra_compile_args=['-std=c++2a'],
    swig_opts=['-c++'],
)

# BINARY DISTRIBUTION(with a Python wrapper)


class BinaryDistribution(dist.Distribution):
    def has_ext_modules(modules):
        return True


# openmtpk Python API modules
modules = [arithmetic, calculus, ml, linalg, nt]

# use the project readme as the description
with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

# def build_pkg():
setuptools.setup(
    name='openmtpk',
    distclass=BinaryDistribution,
    # get version from PyPI package, use bump to increment
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    author='Akiel Aries',
    author_email='akiel@akiel.org',
    description='openMTPK Python API',
    keywords='openmtpk, example, pypi, package',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/akielaries/openMTPK',
    project_urls={
        'Documentation': 'https://akielaries.github.io/openMTPK/',
        'Bug Reports':
        'https://github.com/akielaries/openMTPK/issues',
        'Source Code': 'https://github.com/akielaries/openMTPK/',
    },
    package_dir={'': '.'},
    packages=['openmtpk',
              'openmtpk.arithmetic',
              'openmtpk.calculus',
              'openmtpk.linalg',
              'openmtpk.ml',
              'openmtpk.nt',
              ],
    ext_modules=modules,
    include_package_data=True,
    classifiers=[
        # see https: //pypi.org/classifiers/
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Information Technology',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Topic :: Education',
        'Topic :: Education :: Testing',
        'Topic :: Documentation',
        'Programming Language :: C++',
        'Programming Language :: C',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Fortran',
        'Programming Language :: R',
        'Programming Language :: OCaml',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    python_requires='>=3.6',
    # install_requires = ['Pillow'],
    extras_require={
        'dev': ['check-manifest'],
        # 'test' : ['coverage'],
    },
)



"""
/*************************************************************************
 *
 *  Project
 *                        __  __ _______ _____  _  __
 *                       |  \/  |__   __|  __ \| |/ /
 *  ___  _ __   ___ _ __ | \  / |  | |  | |__) | ' /
 * / _ \| '_ \ / _ \ '_ \| |\/| |  | |  |  ___/|  <
 *| (_) | |_) |  __/ | | | |  | |  | |  | |    | . \ 
 * \___/| .__/ \___|_| |_|_|  |_|  |_|  |_|    |_|\_\ 
 *      | |
 *      |_|
 *
 *
 * Copyright (C) Akiel Aries, <akiel@akiel.org>, et al.
 *
 * This software is licensed as described in the file LICENSE, which
 * you should have received as part of this distribution. The terms
 * among other details are referenced in the official documentation
 * seen here : https://akielaries.github.io/openMTPK/ along with
 * important files seen in this project.
 *
 * You may opt to use, copy, modify, merge, publish, distribute
 * and/or sell copies of the Software, and permit persons to whom
 * the Software is furnished to do so, under the terms of the
 * LICENSE file. As this is an Open Source effort, all implementations
 * must be of the same methodology.
 *
 *
 *
 * This software is distributed on an AS IS basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied.
 *
 ************************************************************************/

openMTPK Calculus Module
=============================
The Calculus Module... TODO

    USAGE :
        >> import openmtpk
        OR
        >> from openmtpk import calculus
"""

from openmtpk.calculus.calculus import (
    Calculus,
)

__all__: list[str]
__path__: list[str]
